#!/bin/sh

check_prereqs()
{
  # Make sure prerequisites are installed
  for i in ffmpeg dcop
  do 
  	which $i >/dev/null 2>&1 || { echo "Could not find pre-requisite: $i" >&2; exit 1; }
  done
}

fix_percent_encoding()
{
  # Hack to fix simple URL escapes
  SEDSCRIPT=/tmp/transcode.$$.sed
  cat <<"EOF">$SEDSCRIPT
  s,%20, ,gi
  s,%21,!,gi
  s,%2A,*,gi
  s,%23,#,gi
  s,%24,$,gi
  s,%26,&,gi
  s,%27,',gi
  s,%28,(,gi
  s,%29,),gi
  s,%2B,+,gi
  s:%2C:,:gi
  s,%3A,:,gi
  s,%3B,;,gi
  s,%3D,=,gi
  s,%3F,?,gi
  s,%40,@,gi
  s,%5B,[,gi
  s,%5D,],gi
  s,%25,%,gi
EOF
  sed -f $SEDSCRIPT  
  rm -f $SEDSCRIPT
}

uri_to_file()
{
   # Only support file URLs.
   case $1 in 
   /*)       echo "$1";;
   file://*) echo "$1" | sed s,^file://,,g | fix_percent_encoding;;
   *)        error $uri "Unsupported/Invalid URI type \"$1\""
             return 1;; 

  esac
  return 0
}

uri_to_short_name()
{
    filename=`uri_to_file $1`
    echo $filename | sed 's,^.*/\([^/]*\)$,\1,'
}

uri_to_xcoded_uri()
{
    uri=$1
    format=$2
    echo $uri | sed "s,\.[^.]*$,.$format,"
}

message()
{
    dcop amarok playlist shortStatusMessage "$*"
}

popup_message()
{
    dcop amarok playlist popupMessage "$*"
}

error()
{
    uri=`shift`
    popup_message "Error transcoding $uri: $*"
    dcop amarok mediabrowser transcodingFinished $uri "\"\""
}

transcode()
{
  command="$1"
  uri="$2"
  format="$3"
  display_result="$4"

  message "Transcoding \"`uri_to_short_name $uri`\" to $format..."
  # Check parameters
  [ $command = "transcode" ] || return 1
  [ $uri = "" ] && return 1
  [ $format = "" ] && return 1

#encoder=`get_encoder "$uri" "$format"` || return 1 
  file=`uri_to_file "$uri"` || return 1
  outfile=`uri_to_xcoded_uri "$file" "$format"` || return 1

  if [ x"$file" = x"$outfile" ];
  then
      error $uri "Transcoded file would overwrite original file.  Try a different format";
      return 1;
  fi
  error=`ffmpeg -i "$file" "$outfile" 2>&1`
  if [ $? -eq 0 ]; then
      dcop amarok mediabrowser transcodingFinished $uri "`uri_to_xcoded_uri $uri $format`"
      if [ x"$display_result" != x ];
      then
          popup_message "Transcoding to $format complete. File written to \"$outfile\""
      fi
      return 0
  else
      error $uri "ffmpeg failed:$error"
      return 1
  fi
}


# Main function
check_prereqs

while read line
do
case "$line" in
  transcode*) transcode $line;;
  *)           echo "Invalid command: $line"  >&2;;
esac
done
